#I "EoS/lib/eos/"
#r "System.Xml.Linq"
#r "EoS.Core"
#r "EoS.CommandLine"

open System
open Microsoft.FSharp.Data.UnitSystems.SI.UnitSymbols
open EoS.Chemistry
open EoS.Phases
open EoS.CommandLine

let database = Flag.DatabaseOpt "db" "Phase database"
let pressure = Flag.PressureRange "P" "Pressure range"
let temperature = Flag.TemperatureRange "T" "Temperature range"
let plotrange = Flag.StringOpt "R" null "Pressure plotting range"
let plotproj = Flag.StringOpt "J" "X15c" "Plot projection setting"

let lines fields (input:IO.TextReader) =
  let rec more () =
    match input.ReadLine() with
    | null ->
      input.Close()
      Seq.empty
    | line ->
      let line =
        let pos = line.IndexOf('#')
        (if pos >= 0 then line.[0 .. pos-1] else line).Trim()
      if String.IsNullOrEmpty(line) then
        more ()
      else
        seq { yield line.Split((null : char[]), fields, StringSplitOptions.RemoveEmptyEntries)
              yield! more () }
  more ()

let getIndex (vs : float<'U>[]) (v : float<'U>) truncate : int =
  let i = Array.BinarySearch(vs, v)
  if i >= 0 then
    i
  else
    let i = ~~~i
    if i > 0 then
      if truncate || i >= vs.Length || abs(v - vs.[i]) >= abs(v - vs.[i - 1]) then
        i - 1
      else
        i
    else
      0

let main args =
  let ps = Seq.toArray pressure.Values
  let Ts = Seq.toArray temperature.Values
  let xs = Array2D.zeroCreate ps.Length Ts.Length

  let phases, track =
    match args with
    | [|phases; xfield; colored; element; track|] ->
      eprintfn "# Obtaing phase objects %A..." phases
      let phases = database.Value.TryGetObject<PhaseCollection>(phases).Value
      let phaseidx = Collections.Generic.Dictionary<IPhase, int>()
      phases |> Seq.iteri (fun i it -> phaseidx.Add(it.Phase, i))

      eprintfn "# Reading grid from %A..." xfield
      for x in IO.File.OpenText(xfield) |> lines Int32.MaxValue do
        let x = Array.map float x
        let p = x.[0] * 1.0<Pa>
        let T = x.[1] * 1.0<K>
        let x = x.[3..]
        let ip, iT = getIndex ps p false, getIndex Ts T false
        xs.[ip, iT] <- x

      eprintfn "# Reading phases from %A..." colored
      let colored =
        [|
          for x in IO.File.OpenText(colored) |> lines 2 do
            let phase = database.Value.TryGetObject<IPhase>(x.[0]).Value
            let color = x.[1]
            yield phaseidx.[phase], color, string phase
        |]

      eprintfn "# Obtaining element object %A..." element
      let element = Formula.ofString element :?> Element

      eprintfn "# Reading track from %A..." track
      let track =
        [|
          for x in IO.File.OpenText(track) |> lines 3 do
            let p = float x.[0] * 1.0<Pa>
            let T = float x.[1] * 1.0<K>
            let ip, iT = getIndex ps p true, getIndex Ts T true
            let y = Array.create phaseidx.Count 0.0
            if xs.[ip, iT] <> null then
              phases.XFold (fun more phase ni xi ->
                let ne =
                  match phase.Formula(xi).Flatten(ni).TryGetValue(element) with
                  | true, ne -> ne
                  | false, _ -> 0.0
                (phase, ne) :: more) List.empty xs.[ip, iT]
              |> Seq.iter (fun (phase, ne) ->
                y.[phaseidx.[phase]] <- ne)
            yield p, T, y
        |]

      colored, track

    | _ ->
      failwith "Need all phases, xfield, selected phases, element and track arguments"

  let J =
    "-J" + plotproj.Value
  let R =
    if plotrange.IsSet then
      if plotrange.Value.Split('/').Length > 2 then
        "-R" + plotrange.Value
      else
        sprintf "-R%s/0/1.01" plotrange.Value
    else
      sprintf "-R%g/%g/0/1.01" (ps.[0]/1.0e9<Pa>) (ps.[ps.Length-1]/1.0e9<Pa>)

  Process.run "psbasemap" [J; R; "-Ba5f1:P/GPa:/a0.1f0.05:phase fraction (atomic):"; "-K"]

  let fractionTo (x:float[]) i =
    (Seq.sum <| seq {
      for j in 0 .. i do
        let ix, _, _ = phases.[j]
        yield x.[ix]
    }) / (Array.sum x)

  for i in 0 .. phases.Length-1 do
    let ix, color, blurb = phases.[i]
    eprintfn "# [%d] %s" ix blurb
    use proc = Process.popen "w" "psxy" [J; R; "-Wthick," + color; "-O"; "-K"]
    let output = proc.StandardInput
    let mutable start = true
    for j in 0 .. track.Length-1 do
      let p, T, x = track.[j]
      if x.[ix] > 0.0 then
        if start then
          output.WriteLine(">")
          if j > 0 then
            let p, T, x = track.[j-1]
            fprintfn output "%g\t%g" (p/1.0e9<Pa>) (fractionTo x (i-1))
          start <- false
        fprintfn output "%g\t%g" (p/1.0e9<Pa>) (fractionTo x i)
      elif not start then
        fprintfn output "%g\t%g" (p/1.0e9<Pa>) (fractionTo x (i-1))
        start <- true
    Process.wait proc

  do
    use proc = Process.popen "w" "pslegend" [J; R; "-Dx22.5/11.25/5/7.5/ML"; "-O"]
    let output = proc.StandardInput
    for _, color, blurb in phases do
      fprintfn output "S 0 - 0.5 - thick,%s 0.5 %s" color blurb
    Process.wait proc

  0

Flag.WithCommandLineArgs main
