#!/usr/bin/env python2
# -*- encoding: utf-8 -*-

"""Elastic property conversion into final format"""
from __future__ import division, print_function
from bisect import bisect
from os import path
from getopt import getopt
import sys, re


def read_data(fi):
    """Read floating point data from a stream of lines.
    :type fi: iter
    :rtype: iter
    """
    comment = re.compile(r"#.*$")
    for line in fi:
        fields = map(float, comment.sub("", line).strip().split())
        if fields:
            yield fields


prem = [[], []]
"""PREM radius and pressure values"""


with open(path.join(path.dirname(sys.argv[0]), "prem.txt"), 'r') as fi:
    for x in read_data(fi):
        prem[0].append(x[0])
        prem[1].append(x[7])


def prem_r2p(r):
    """Extract the pressure corresponding to a radius value in PREM.
    Also returns dp/dr.
    :type r: float
    :rtype: float, float
    """
    i1 = bisect(prem[0], r)
    if i1 <= 0:
        i0 = 0
        i1 = 1
    elif i1 >= len(prem[0]):
        i0 = -2
        i1 = -1
    else:
        i0 = i1 - 1

    r0 = prem[0][i0]
    r1 = prem[0][i1]

    p0 = prem[1][i0]
    p1 = prem[1][i1]

    dp_dr = (p1 - p0) / (r1 - r0)
    return p0 + dp_dr * (r - r0), dp_dr


def prem_d2p(z):
    """Extract the pressure corresponding to a depth value in PREM.
    Also returns dp/dz.
    :type z: float
    :rtype: float, float
    """
    p, dp_dr = prem_r2p(prem[0][-1] - z)
    return p, -dp_dr


if __name__ == '__main__':
    convert = prem_r2p
    convert_dx = False

    opts, args = getopt(sys.argv[1:], 'rzd', ['dp'])
    for o, a in opts:
        if o == '-r':
            convert = prem_r2p
        elif o in ('-z', '-d'):
            convert = prem_d2p
        elif o == '--dp':
            convert_dx = True

    def write_output(x):
        p, dp_dr = convert(x[0])
        if convert_dx:
            print(p, *map(lambda xi: xi / dp_dr, x[1:]))
        else:
            print(p, *x[1:])

    if args:
        for a in args:
            with open(a, 'r') as fi:
                for x in read_data(fi):
                    write_output(x)
    else:
        for x in read_data(sys.stdin):
            write_output(x)
